# Generated by Django 4.0.4 on 2022-05-03 07:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='HgncGene',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('hgnc_id', models.CharField(db_index=True, max_length=50)),
                ('approved_symbol', models.CharField(db_index=True, max_length=100)),
                ('alias_symbols', models.CharField(blank=True, db_index=True, max_length=255, null=True)),
                ('approved_name', models.CharField(max_length=250)),
                ('alias_names', models.CharField(blank=True, max_length=250, null=True)),
                ('chromosome_location', models.CharField(blank=True, db_index=True, max_length=50, null=True)),
                ('orphanet_id', models.CharField(blank=True, db_index=True, max_length=50, null=True)),
                ('mim_number', models.CharField(blank=True, db_index=True, max_length=50, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='OmimGene',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('mim_number', models.CharField(db_index=True, max_length=50)),
                ('gene_symbols', models.CharField(db_index=True, max_length=100)),
                ('chromosome_location', models.CharField(blank=True, db_index=True, max_length=50, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='OmimTitle',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('prefix', models.CharField(blank=True, db_index=True, max_length=50, null=True)),
                ('preferred', models.TextField()),
                ('alternative', models.TextField()),
                ('omim_gene', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='genetics.omimgene')),
            ],
        ),
        migrations.CreateModel(
            name='OmimPhenotype',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('phenotype', models.CharField(max_length=250)),
                ('omim_gene', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='phenotypes', to='genetics.omimgene')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from django.db import models


class HgncGene(models.Model):
    """
    Importo solo gli approved
    """
    hgnc_id = models.CharField(max_length=50, db_index=True)
    approved_symbol = models.CharField(max_length=100, db_index=True)
    alias_symbols = models.CharField(max_length=255, db_index=True, null=True, blank=True)
    approved_name = models.CharField(max_length=250)
    alias_names = models.CharField(max_length=250, null=True, blank=True)
    chromosome_location = models.CharField(max_length=50, db_index=True, null=True, blank=True)
    orphanet_id = models.CharField(max_length=50, db_index=True, null=True, blank=True)  # non viene più esportato ?
    mim_number = models.CharField(max_length=50, db_index=True, null=True, blank=True)  # omim_id

    def __str__(self):
        return "%s - %s" % (self.approved_symbol, self.approved_name)

    @property
    def omim_phenotypes(self):
        phenotypes = []
        for op in OmimPhenotype.objects.filter(omim_gene__mim_number=self.mim_number).distinct():
            phenotypes.append(op.phenotype)
        return phenotypes

    @property
    def omim_titles(self):
        titles = OmimTitle.objects.filter(omim_gene__mim_number=self.mim_number)
        if titles.exists():
            # there can be only one
            return str(titles[0])
        return ""

    @property
    def html_omim_titles(self):
        titles = OmimTitle.objects.filter(omim_gene__mim_number=self.mim_number)
        if titles.exists():
            # there can be only one
            return titles[0].html()
        return ""

    def str_html(self):
        return '<a href="https://omim.org/entry/%s" target="_blank">%s</a>' % (self.mim_number, self)

    class Meta:
        app_label = 'genetics'


class OmimGene(models.Model):
    mim_number = models.CharField(max_length=50, db_index=True)  # MIM Number
    gene_symbols = models.CharField(max_length=100, db_index=True)
    chromosome_location = models.CharField(max_length=50, db_index=True, null=True, blank=True)

    class Meta:
        app_label = 'genetics'


class OmimPhenotype(models.Model):
    omim_gene = models.ForeignKey(OmimGene, on_delete=models.CASCADE, related_name='phenotypes')
    phenotype = models.CharField(max_length=250)

    class Meta:
        app_label = 'genetics'


class OmimTitle(models.Model):
    prefix_map = {
        "NULL": '',
        "Number Sign": '# ',
        "Plus": '+ ',
        "Caret": '^ ',
        "Asterisk": '* ',
        "Percent": '% '
    }
    prefix = models.CharField(max_length=50, db_index=True, null=True, blank=True)
    omim_gene = models.ForeignKey(OmimGene, on_delete=models.CASCADE)
    preferred = models.TextField()
    alternative = models.TextField()

    @property
    def one_char_prefix(self):
        try:
            return OmimTitle.prefix_map[self.prefix]
        except:
            return ''

    def __str__(self):
        if self.alternative:
            return "%sPREF: %s ALT: %s" % (self.one_char_prefix, self.preferred, self.alternative)
        else:
            return "%s%s" % (self.one_char_prefix, self.preferred)

    def html(self):
        if self.alternative:
            return "<strong>%sPREF</strong>: %s <strong>ALT</strong>: %s" % (
                self.one_char_prefix, self.preferred, self.alternative)
        else:
            return "<strong>%s</strong>%s" % (self.one_char_prefix, self.preferred)

    class Meta:
        app_label = 'genetics'
